/* \file messages.c
 */

#include <stdio.h>
#include <unistd.h>
#include <string.h>
#include "messages.h"
#include "utils.h"

void MSG_Header(int sock,int code,const char *message) {
  char buf[10];

  writeString(sock,"HTTP/1.0 "); 
  sprintf(buf,"%d ",code);
  writeString(sock,buf);
  writeString(sock,message);
  writeString(sock,"\r\n\r\n");
}
 
 
void MSG_Body(int sock,const char *message) {
  writeString(sock,"<HTML><H1>");
  writeString(sock,message);
  writeString(sock,"</H1></HTML>\n");
}
 
 
void MSG_PageHeader(int sock) {
  writeString(sock,"<html><head><title>WebControl</title></head>\n"
                   "<body bgcolor=\"#FFFFFF\">\n");
  writeString(sock,"<table border=\"0\" cellpadding=\"0\""
                   " cellspacing=\"0\" width=\"100%\">");
  writeString(sock,"<tr><td valign=\"top\">"
                   "<h1 align=\"left\">"
                   "<font color=\"#0000FF\">Web</font>Control</h1>");
  writeString(sock,"</td>\n<td align=\"right\" valign=\"top\">"
                   "<a href=\"http://www.coyotelinux.com/\""
                   " style=\"text-decoration:none\">"
                   "<font color=\"black\">Coyote Linux</font>"
                   "<a>"
                   "</td>");
  writeString(sock,"</tr></table>\n");
}
 
 
void MSG_PageFooter(int sock) {
  writeString(sock,"\n<hr><p><em>WebControl v" WEBCTL_VERSION ", by </em>");
  writeString(sock,"<a href=\"mailto:Michael.McTernan.2001@cs.bris.ac.uk\">"
                   "<em>Michael McTernan</em></a></p>\n");
  writeString(sock,"</body></html>\n");
}

void MSG_Error(int sock,int code,const char *message) {
  MSG_Header(sock,code,message);
  MSG_PageHeader(sock);
  MSG_Body(sock,message);
  MSG_PageFooter(sock);
}

/*MSG_SetDescriptors*/
int *MSG_SetDescriptors(int sock) {
  int *r,t;

  /* Make array */
  r=malloc(sizeof(int)*3);
  critical(r!=NULL);

  /* Save and replace fds */
  for(t=0;t<3;t++) {

    r[t]=dup(t);
    dup2(sock,t);

  }

  return r;

}

/*MSG_ResetDescriptors*/
void MSG_ResetDescriptors(int *r) {
  int t;  

  for(t=0;t<3;t++) {
  
    /* Restore fds */
    dup2(r[t],t);
 
    /* Close off the duplicates */
    close(r[t]);

  }

  free(r);
}

