/* dnslookup.c
 *
 * Minimal pseudo-cgi that offers a DNS lookup service.
 * Copyright (C) 2001 Michael McTernan,
 *   Michael.McTernan.2001@cs.bris.ac.uk
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <netdb.h> 
#include <stdlib.h>
#include <string.h>
#include <stdio.h>


int main(int argC,char **argV) {
  char *query,*host;
  int   a;

  printf("<h2>DNS Resolver</h2>\n");

  /* Get the query string */
  query=getenv("QUERY_STRING");

  /* Check if the query has the required params */
  if(query!=NULL && (host=strstr(query,"="))!=NULL) {
    struct hostent *hent;

    /* Skip the '=' sign */
    host++;

    /* Look it up */
    hent=gethostbyname(host);
    if(hent!=NULL)
      hent=gethostbyaddr(hent->h_addr_list[0],sizeof(struct in_addr),AF_INET);

    if(hent==NULL) {
    
      printf("<p>Failed to lookup hostname <b>%s</b></p>\n",host);
      printf("<ul><li><a href=\"dnslookup.c\">Resolve another</a>");
      printf("<li><a href=\"/\">Main menu</a></ul>");

    } else {

      printf("<table border=\"0\"><tr><td>Name:</td><td>%s</td></tr>\n",
             hent->h_name);

      /* Aliases */
      if(hent->h_aliases[0]!=NULL) {
        
        printf("<tr><td valign=\"top\">Aliases:</td><td>");
        
        a=0;
        while(hent->h_aliases[a]!=NULL) {   
          printf("%s<BR>",hent->h_aliases[a]);
          a++;
        }

        printf("</td></tr>\n");
      }        

      /* IP Addresses */
      printf("<tr><td valign=\"top\">IP Address:</td><td>");
      a=0;
      while(hent->h_addr_list[a]!=NULL) {
        struct in_addr inaddr;
        char *addr;

        memcpy(&inaddr,hent->h_addr_list[a],hent->h_length);
        addr=inet_ntoa(inaddr);

        printf("%s<br>",addr);
        a++;
      }

      printf("</td></tr></table>\n");

      printf("<ul><li><a href=\"dnslookup.c\">Resolve another</a>");
      printf("<li><a href=\"/\">Main menu</a></ul>");

    }

  } else {

    printf("<p>Enter the host name or IP address that you wish to resolve.\n"
           "<form method=\"GET\">"
           "<input type=\"text\" name=\"q\">\n"
           "<input type=\"submit\" value=\"Ok\">\n"
           "</form>\n"
           "</p>\n");

     printf("<ul><li><a href=\"/\">Main menu</a></ul>");        
  }

  

  return 0;
}

