#!/bin/sh

. /etc/coyote/coyote.conf

foot () {
  echo "<ul><li><a href=\"http://$LOCAL_IPADDR:888/\">Main menu</a></ul>"
}

echo "<h2>Change Network Parameters</h2>"

if [ -z "$QUERY_STRING" ]; then
  echo "<p>Malformed request</p>"
  foot
  exit
fi

parse() {
  VAL=`echo $QUERY_STRING | sed "s/.*$PARAM=\([^&]*\)\(&.*\|$\)/\1/"`
}

PARAM=ip; parse; IPADDR=$VAL;
PARAM=mask; parse; NETMASK=$VAL;
PARAM=broadcast; parse; BROADCAST=$VAL;
PARAM=dns1; parse; DNS1=$VAL;
PARAM=dns2; parse; DNS2=$VAL;

CMD="ifconfig $IF_LOCAL"


echo -e ". /etc/coyote/coyote.conf\nrm -f /etc/coyote/coyote.conf" > /tmp/coyote.rebuild 

if [ -n "$IPADDR" ]; then
  CMD="$CMD $IPADDR"
  echo "LOCAL_IPADDR=$IPADDR" >> /tmp/coyote.rebuild 
else
  CMD="$CMD $LOCAL_IPADDR"
fi

if [ -n "$NETMASK" ]; then
  CMD="$CMD netmask $NETMASK"
  echo "LOCAL_NETMASK=$NETMASK"  >> /tmp/coyote.rebuild 
else
  CMD="$CMD netmask $LOCAL_NETMASK"
fi

if [ -n "$BROADCAST" ]; then
  CMD="$CMD broadcast $BROADCAST"
  echo "LOCAL_BROADCAST=$BROADCAST"  >> /tmp/coyote.rebuild 
else
  CMD="$CMD broadcast $LOCAL_BROADCAST"
fi  

(

[ -n "$DNS1" ] && echo "DNS1=$DNS1" 
[ -n "$DNS2" ] && echo "DNS2=$DNS2"

echo "("
cat /etc/coyote/coyote.conf | sed "s/\([^=]*\).*/echo \"\1=\\\\\"$\1\\\\\"\"/"
echo ") > /etc/coyote/coyote.conf"

) >> /tmp/coyote.rebuild

# Implement changes 
echo "domain $DOMAINNAME" > /etc/resolv.conf
echo "nameserver $DNS1" >> /etc/resolv.conf
echo "nameserver $DNS2" >> /etc/resolv.conf


if [ -x /usr/sbin/mkdproxyconf.sh ]; then
  echo "<p>Updating DNS cache server settings.</p>"
  echo "DNS1=$DNS1" >> /etc/coyote/coyote.conf
  /usr/sbin/mkdproxyconf.sh
  DPROXY=`pidof dproxy`
  if [ -n "$DPROXY" ]; then
    echo "<p>Restarting DNS cache server.</p>"
    (kill $DPROXY
     sleep 4
     /usr/sbin/dproxy -c /etc/dproxy.conf ) 2> /dev/null
  fi

fi

if [ "$DHCP_SERVER" = "YES" ]; then
  echo "<p>Updating DHCP settings.</p>"
  /usr/sbin/mkdhcpconf.sh
  DHCPD=`pidof dhcpd`
  if [ -n "$DHCPD" ]; then
    echo "<p>Restarting DHCP daemon.</p>"
    (kill $DHCPD 
    sleep 4
    dhcpd -q $IF_LOCAL ) 2> /dev/null
  fi
fi


$CMD 

if [ $? = 0 ]; then
  echo "<p>Changes suceeded.</p>"

  . /tmp/coyote.rebuild
  touch /etc/coyote/coyote.conf.dirty
else
  echo "<p>Changes failed.  Check your parameters and try again.</p>"
fi

rm -f /tmp/coyote.rebuild

foot

