#!/bin/sh

. /etc/coyote/coyote.conf

echo "<h2>PPP Status</h2>"  

writeupdate() {
  if [ -n "$QUERY_STRING" ]; then
    echo "<META HTTP-EQUIV=\"refresh\" content=\"10;URL=?update=5\">"
  fi
}

writestopmenu() {
  echo "<ul><li><a href=\"ppp-down.cgi\">Stop PPPD</a><br><br>"
  echo "<li><a href=\"/\">Main menu</a></ul>"
}

writestartmenu() {
  echo "<ul>"
  ls /etc/ppp/*.env |  sed "s/\/.*\/\([^\.]*\)\.env/<li><a href=\"ppp-up.cgi?isp=\1\">Connect to '\1'<\/a>/"
  echo "<br><br><li><a href=\"/\">Main menu</a></ul>"
}

writelastmessages() {
  echo "<h2>Messages</h2>"
 
  echo "<p>These are the messages logged from the last instantiation of pppd"
  echo "and chat.</p>"
 
  echo "<h3>PPP</h3><pre>"
  LASTPPP=`grep pppd /var/log/daemon.log | tail -n 1 | sed "s/.* pppd\[\([0-9]*\)\]: .*/\1/"`
  grep "pppd[$LASTPPP]: " /var/log/daemon.log
  echo "</pre>"
 
  echo "<h3>Chat</h3><pre>"
  LASTCHAT=`grep chat /var/log/messages | tail -n 1 | sed "s/.* chat\[\([0-9]*\)\]: .*/\1/"`
  grep "chat[$LASTCHAT]: " /var/log/messages
  echo "</pre>"
}

writeconnspeed() {
   LASTCHAT=`grep chat /var/log/messages | tail -n 1 | sed "s/.* chat\[\([0-9]*\)\]: .*/\1/"`
  CONNSPEED=`grep " chat[$LASTCHAT]: " /var/log/messages | sed "s/^.*: *\([0-9]*\).*$/\1/"`
  CONNSPEED=`echo $CONNSPEED`

  if [ -n "$CONNSPEED" ]; then
    echo "<p>Connection speed: <b>$CONNSPEED</b></p>"

    if [ "$CONNSPEED" = "115200" ] ; then 
      echo "<p><i>Note: It appears that your modem is reporting the connection"
      echo "speed with the computer, not the remote modem.  This means that the"
      echo "actual speed of the connection with the ISP may be lower.  To fix"
      echo "this, you may try adding <B>W2</B> to the end of your modem init"
      echo "string, or refer to you modem guidebook for instructions.</i></p>"
    fi
  fi
}

PPP=`pidof pppd`
CHAT=`pidof /usr/sbin/chat`
ROUTE=`route -n | grep $IF_INET`
DEMAND=`grep demand /etc/ppp/options 2> /dev/null`


if [ -z "`ls -l /etc/ppp/*.env 2> /dev/null`" ]; then
  echo "<p>No connections to ISPs have yet been created.  You must do this "
  echo "before you will be able to connect to the internet.</p>"
  echo "<ul><li><a href=\"/cgi-bin/ppp-config-menu.cgi\">Configure connections</a><br><br>"
  echo "<li><a href=\"/\">Main menu</a></ul>"  
  exit
fi

if [ -z "$PPP" ]; then

  echo "<p>The pppd daemon is not running, meaning that you are not connected"
  echo "to the internet through a dialup link.</p>"

  LASTCHAT=`grep chat /var/log/messages | tail -n 1 | sed "s/.* chat\[\([0-9]*\)\]: .*/\1/"`
  CHATFAIL=`grep "chat[$LASTCHAT]: " /var/log/messages | tail -n 1 | sed "s/.*chat\[$LASTCHAT\]: Failed (\([^)]*\)).*/\1/"` 

  case "$CHATFAIL" in
  "NO DIALTONE")
    echo "<p><i>Chat indicates that the last attempt to connect failed because"
    echo "there was no dialing tone.  If you are experiencing problems"
    echo "connecting, you should check that the modem is plugged into the"
    echo "phone line, and that the phone line has a dialing tone.  In cases"
    echo "where an exchange is used, it may be necessary to include"
    echo "an access number to access an external line." 
    echo "</i></p>"

    echo "<p>Phone number dialed: "
    grep "chat[$LASTCHAT]: " /var/log/messages | grep ATDT | sed "s/^.*chat\[$LASTCHAT\]: .*ATDT\([0-9 ]*\).*$/\1/" | head -n 1
    echo "</p>"
    ;;
  "NO CARRIER")
    echo "<p><i>Chat indicates that the last attempt to connect failed because"
    echo "there was no carrier detected.  This means that the modem failed to"
    echo "establish communications with the remote end, and that you should"
    echo "check that you are dialing the correct phone number for your"
    echo "Internet service provider.</i></p>" 
    ;;
  "BUSY")
    echo "<p><i>Chat indicates that the last attempt to connect failed because"
    echo "the number that was dialled is engaged.  You may wish to verfiy that"
    echo "you are dialing the correct phone number for your"
    echo "Internet service provider, or change ISP if this is a common problem."
    echo "</i></p>"
    
    echo "<p>Phone number dialed: "
    grep "chat[$LASTCHAT]: " /var/log/messages | grep ATDT | sed "s/^.*chat\[$LASTCHAT\]: .*ATDT\([0-9 ]*\).*$/\1/" | head -n 1
    echo "</p>"

    ;;
  "Invalid Login" | "Login incorrect")
    echo "<p><i>Chat indicates that the last attempt to connect failed because"
    echo "the username and password combination was rejected when logging into"
    echo "your Internet service provider.  This maybe due to either the"
    echo "username or password being used being incorrect, or entered"
    echo "incorrectly.  You should verify that you entered the correct"
    echo "credentials, or check with your ISP if uncertain.</i></p>" 
    ;;
  esac

  writestartmenu
  writelastmessages

  exit 

fi 

if [ -n "$CHAT" ]; then

  writeupdate

  echo "<p>PPPD and CHAT are running.  It is likely that the modem is"
  echo "dialing, and that the initial connection with the isp is being"
  echo "negotiated.</p>"

  writestopmenu
  writelastmessages

  exit 

fi

if [ -n "$DEMAND" ]; then

  echo "<p>PPPD is running and configured for demand dialing."
  echo "This means that pppd will decide when to connect and disconnect"
  echo "to your ISP, based upon traffic that it sees."
  echo "Stopping pppd will prevent it from connecting to the Internet.</p>" 

  writestopmenu
  writelastmessages 

  exit

fi

if [ -z "$ROUTE" ]; then

  writeupdate

  echo "<p>PPPD is running, but there is no route yet established that uses"
  echo "a point-to-point internet interface.  It is likely that PPPD is"
  echo "negotiating link parameters with your isp and is imminent to completing"
  echo "the connection.</p>"
  
  writeconnspeed
  writestopmenu
  writelastmessages

  exit 

fi


echo "<p>PPPD is running, and there is a route established that uses the"
echo "point-to-point interface.  You are most probably connected to the"
echo "internet!</p>"

writeconnspeed

if [ -n "$WEBCTL_USER_HOMEPAGE" ]; then

  if [ -n "$QUERY_STRING" ]; then
 
    echo "<META HTTP-EQUIV=\"refresh\" content=\"3;URL=$WEBCTL_USER_HOMEPAGE\">"
 
  fi

  echo "<ul><li><a href=\"$WEBCTL_USER_HOMEPAGE\">$WEBCTL_USER_HOMEPAGE</a></ul>"

fi

writestopmenu
writelastmessages
 

 
