/* timedread.c 
 *
 * Wrapper for read(2) but with a timeout facility.
 * Copyright (C) 2000 Michael McTernan
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

#include <sys/types.h>
#include <sys/time.h>
#include <unistd.h>
#include <errno.h>
#include "timedread.h"

/*timedread*/
/* See read(2) for description of functionality.
 * Returns -1 on error and sets errno.
 */
int timedread(int fd,void *buf,size_t count) {
  struct timeval timeout;
  fd_set readSet,writeSet,exceptSet;
  int r;

  /* Create the sets */
  FD_ZERO(&readSet);
  FD_ZERO(&writeSet);
  FD_ZERO(&exceptSet);

  FD_SET(fd,&readSet);
  FD_SET(fd,&exceptSet);

  /* 12 second timeout */
  timeout.tv_sec=12;
  timeout.tv_usec=0;

  /* Select */
  r=select(fd+1,&readSet,&writeSet,&exceptSet,&timeout);
  if(r==0 || !FD_ISSET(fd,&readSet)) {
    errno=ETIMEDOUT;
    return -1;
  }

  if(FD_ISSET(fd,&exceptSet)) {
    errno=EIO;
    return -1;
  }

  return read(fd,buf,count);
}
