/* user.c
 *
 * Copyright (C) 2001 Michael McTernan,
 *   Michael.McTernan.2001@cs.bris.ac.uk
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

#include <sys/types.h>
#include <sys/stat.h>
#include <sys/wait.h>
#include <stdlib.h>
#include <fcntl.h> 
#include <unistd.h>
#include <string.h>
#include <stdio.h>
#include "user.h"
#include "request.h"
#include "messages.h"
#include "utils.h"

int checkUser(int sock,request_t req) {
  const char *requri;
  char buffer[512];
  struct stat stats;
  int   f,t;
  
  /* Get the URI */
  requri=REQ_GetDemangledURL(req);

  /* Check for buffer violation */
  if(strlen(requri)+strlen(USER_ROOT)>sizeof(buffer)-1) {
    MSG_Error(sock,404,"Not Found");
    return -1;
  }

  /* For the whole path */
  strcpy(buffer,USER_ROOT);
  strcat(buffer,&requri[strlen(USER_WWW_VROOT)]);

  /* Check if we need to look at index.html */
  if(strcmp(buffer,USER_ROOT)==0) 
    strcat(buffer,"index.html");

  /* Check if the file exists */
  if(stat(buffer,&stats)==-1) {
    MSG_Error(sock,404,"Not Found");
    return -1;
  }

  /* Check the file is normal */
  if((stats.st_mode & S_IFREG)==0) {
    MSG_Error(sock,403,"Forbidden");  
    return -1;  
  }

  /* Is it a cgi? */
  if(strncmp(USER_WWW_VROOT "cgi-bin/",requri,9)==0) {

    /* Is the file world readable & executable? */
    if((stats.st_mode & S_IXOTH)==0 || (stats.st_mode & S_IROTH)==0) { 
      MSG_Error(sock,403,"Forbidden");
      return -1;
    }

    /* Execute as a new process */
    if(fork()==0) {
      int  *fds; 

      /* Set the QUERY_STRING env variable */
      if(REQ_GetQuery(req)!=NULL)
        setenv("QUERY_STRING",REQ_GetQuery(req),1);
      else
        setenv("QUERY_STRING","",1);

      /* Set stdin/out etc */
      fds=MSG_SetDescriptors(sock);

      /* Send first line of header */
      writeString(sock,"HTTP/1.0 200 Ok\r\n");

      /* Append to path if needed */
      #ifdef USER_CGI_PATH_APPEND
      {
        char *pth,*newpth;

        pth=getenv("PATH");
        newpth=malloc(strlen(pth)+strlen(USER_CGI_PATH_APPEND)+1);
        strcpy(newpth,pth);
        strcat(newpth,USER_CGI_PATH_APPEND);

        setenv("PATH",newpth,1);
        free(newpth);

      }
      #endif

      /* Change root and run */
      #ifdef USER_CGI_CHROOT
        chroot(USER_ROOT);
        system(&buffer[strlen(USER_ROOT)-1]);
      #else
        system(buffer);   
      #endif       

      /* Return descriptors */
      MSG_ResetDescriptors(fds);

      /* Ensure any printf's are flushed */
      fflush(stdout);
      fflush(stderr);
      exit(0);
    }

    return 0;
  }


  /* Not cgi, must just be a file */
  if(fork()==0) {

    /* Safety */
    chroot(USER_ROOT);   

    /* Attempt to open file (earlier stat was ok, should not fail) */
    f=open(&buffer[strlen(USER_ROOT)-1],O_RDONLY);

    MSG_Header(sock,200,"Ok");

    /* Copy file contents to socket */
    do {
      t=read(f,buffer,sizeof(buffer));
      if(t>0) 
        write(sock,buffer,t);

    } while(t>0);

    close(sock);
    close(f);
    exit(0);
  }

  return 0;

}



